@@ -1,12 +1,10 @@
 """The base Simon models"""
 
-from datetime import datetime
-
 from .connection import get_database
 from .exceptions import MultipleDocumentsFound, NoDocumentFound
 from .query import Q, QuerySet
-from .utils import (get_nested_key, guarantee_object_id, map_fields,
-                    remove_nested_key, update_nested_keys)
+from .utils import (current_datetime, get_nested_key, guarantee_object_id,
+                    map_fields, remove_nested_key, update_nested_keys)
 
 __all__ = ('Model',)
 
@@ -27,7 +25,8 @@ class Meta(object):
         self.database = 'default'
         self.field_map = {}
         self.map_id = True
-        self.safe = False
+        self.required_fields = None
+        self.safe = True
         self.sort = None
 
     def add_to_original(self, cls, name):
@@ -47,7 +46,8 @@ class Meta(object):
 
             # Add the known attributes to the instance
             for name in ('auto_timestamp', 'collection', 'database',
-                         'field_map', 'map_id', 'safe', 'sort'):
+                         'field_map', 'map_id', 'required_fields', 'safe',
+                         'sort'):
                 if name in meta_attrs:
                     setattr(self, name, meta_attrs.pop(name))
 
@@ -75,6 +75,14 @@ class Meta(object):
             # If map_id is True and id isn't in field_map, add it.
             self.field_map['id'] = '_id'
 
+        # Any of the methods that check for required fields are looking
+        # for something like a list or tuple of fields, not a string
+        # of one field. If a single field name has been provided as a
+        # string, switch it to a tuple now.
+        if self.required_fields and not isinstance(self.required_fields,
+                                                   (list, tuple)):
+            self.required_fields = (self.required_fields,)
+
         # When calling QuerySet.sort(), it accepts the sort fields as
         # *args. Forcing Meta.sort into an iterable now allows it to
         # be specified as either a single field or a list when defining
@@ -202,16 +210,29 @@ class Model(object):
         instantiate a new ``Model`` from the keyword arguments,
         call ``save()``, and return the instance.
 
+        If the model has the ``required_fields`` options set, a
+        :class:`TypeError` will be raised if any of the fields are not
+        provided.
+
         :param safe: (optional) Whether to perform the create in safe
                      mode.
         :type safe: bool.
         :param \*\*fields: Keyword arguments to add to the document.
         :type \*\*fields: \*\*kwargs.
         :returns: :class:`~simon.Model` -- the new document.
+        :raises: :class:`TypeError`
 
         .. versionadded:: 0.1.0
         """
 
+        if cls._meta.required_fields and any(k not in fields for k
+                                             in cls._meta.required_fields):
+            message = ("The '{0}' object cannot be created because it must "
+                       "contain all of the required fields: {1}")
+            message = message.format(cls.__name__,
+                                     ', '.join(cls._meta.required_fields))
+            raise TypeError(message)
+
         new = cls(**fields)
         new.save(safe=safe)
 
@@ -441,9 +462,16 @@ class Model(object):
         fields = dict((k, 1) for k in update.keys())
         doc = self._meta.db.find_one({'_id': id}, fields)
 
+        # There's no need to update the _id
+        doc.pop('_id')
+
         doc = update_nested_keys(self._document, doc)
 
-        for k, v in doc.items():
+        for k, v in doc.iteritems():
+            # Normally I'd use self._document[k] = v, but if a value
+            # has already been associated with an attribute, assigning
+            # the new value through _document won't update the
+            # attribute's value.
             setattr(self, k, v)
 
     def raw_update(self, fields, safe=False):
@@ -489,6 +517,11 @@ class Model(object):
         # reloaded in order to update the instance.
         doc = self._meta.db.find_one({'_id': self.id})
 
+        # There's no need to update the _id
+        doc.pop('_id')
+
+        doc = update_nested_keys(self._document, doc)
+
         for k, v in doc.iteritems():
             # Normally I'd use self._document[k] = v, but if a value
             # has already been associated with an attribute, assigning
@@ -510,6 +543,10 @@ class Model(object):
         Unlike :meth:`~simon.Model.save`, ``modified`` will not be
         updated.
 
+        If the model has the ``required_fields`` options set, a
+        :class:`TypeError` will be raised if attempting to remove one of
+        the required fields.
+
         :param fields: The names of the fields to remove.
         :type fields: str, list, or tuple.
         :param safe: (optional) Whether to perform the save in safe
@@ -526,6 +563,14 @@ class Model(object):
                             "'{1}' attribute has not been set.".format(
                                 self.__class__.__name__, '_id'))
 
+        if self._meta.required_fields and any(k in fields for k
+                                              in self._meta.required_fields):
+            message = ("The '{0}' object cannot be updated because it must "
+                       "contain all of the required fields: {1}")
+            message = message.format(self.__class__.__name__,
+                                     ', '.join(self._meta.required_fields))
+            raise TypeError(message)
+
         # fields can contain a single item as a string. If it's not a
         # list or tuple, make it one. Otherwise the generators below
         # would iterate over each character in the string rather than
@@ -562,13 +607,26 @@ class Model(object):
         added with the current datetime in UTC. ``modified`` will
         always be set with the current datetime in UTC.
 
+        If the model has the ``required_fields`` options set, a
+        :class:`TypeError` will be raised if any of the fields have not
+        been associated with the instance.
+
         :param safe: (optional) Whether to perform the save in safe
                      mode.
         :type safe: bool.
+        :raises: :class:`TypeError`
 
         .. versionadded:: 0.1.0
         """
 
+        if self._meta.required_fields and any(k not in self._document for k
+                                              in self._meta.required_fields):
+            message = ("The '{0}' object cannot be saved because it must "
+                       "contain all of the required fields: {1}")
+            message = message.format(self.__class__.__name__,
+                                     ', '.join(self._meta.required_fields))
+            raise TypeError(message)
+
         # Associate the current datetime (in UTC) with the created
         # and modified fields. While Python can store datetimes with
         # microseconds, BSON only supports milliseconds. Rather than
@@ -576,8 +634,7 @@ class Model(object):
         # from the Python datetime before associating it with the
         # instance.
         if self._meta.auto_timestamp:
-            now = datetime.utcnow()
-            now = now.replace(microsecond=(now.microsecond / 1000 * 1000))
+            now = current_datetime()
             if not ('_id' in self._document and 'created' in self):
                 self.created = now
             self.modified = now
@@ -717,11 +774,18 @@ class Model(object):
         for k, v in doc.items():
             setattr(self, k, v)
 
-    def __contains__(self, name):
-        """Check for a key in a document"""
+    # String representation methods
 
-        key = self._meta.field_map.get(name, name)
-        return key in self._document
+    def __repr__(self):
+        return '<{0}: {1}>'.format(self.__class__.__name__, self)
+
+    def __str__(self):
+        return '{0} object'.format(self.__class__.__name__)
+
+    def __unicode__(self):
+        return u'{0} object'.format(self.__class__.__name__)
+
+    # Attribute access methods
 
     def __delattr__(self, name):
         """Remove a key from the document"""
@@ -773,9 +837,6 @@ class Model(object):
                 self.__class__.__name__, name))
         return self._document[name]
 
-    def __repr__(self):
-        return '<{0}: {1}>'.format(self.__class__.__name__, self)
-
     def __setattr__(self, name, value):
         """Set a document value"""
 
@@ -792,8 +853,40 @@ class Model(object):
             name = self._meta.field_map.get(name, name)
             self._document[name] = value
 
-    def __str__(self):
-        return '{0} object'.format(self.__class__.__name__)
+    # Rich comparison methods
 
-    def __unicode__(self):
-        return u'{0} object'.format(self.__class__.__name__)
+    def __eq__(a, b):
+        """Check equality of two instances"""
+
+        # If either b isn't of the right type, a and b use different
+        # database connections, or a and b use different collections,
+        # the two cannot be equal.
+        if not (isinstance(a, b.__class__) or isinstance(b, a.__class__)):
+            return False
+        if a._meta.database != b._meta.database:
+            return False
+        if a._meta.collection != b._meta.collection:
+            return False
+
+        a_id = a._document.get('_id', None)
+        b_id = b._document.get('_id', None)
+
+        # If either one of the instances has a value of _id, the two
+        # cannot be equal.
+        if not (a_id and b_id):
+            return False
+
+        return a_id == b_id
+
+    def __ne__(a, b):
+        """Check inequality of two instances"""
+
+        return not a.__eq__(b)
+
+    # Container methods
+
+    def __contains__(self, name):
+        """Check for a key in a document"""
+
+        key = self._meta.field_map.get(name, name)
+        return key in self._document
